Package: scater
Type: Package
Authors@R: c(
        person("Davis", "McCarthy", role=c("aut"), email="davis@ebi.ac.uk"),
        person("Kieran", "Campbell", role=c("aut"), email="kieran.campbell@sjc.ox.ac.uk"),
        person("Aaron", "Lun", role = c("aut", "ctb"), email="infinite.monkeys.with.keyboards@gmail.com"),
        person("Quin", "Wills", role=c("aut"), email="qilin@quinwills.net"),
        person("Vladimir", "Kiselev", role=c("ctb"), email="vk6@sanger.ac.uk"),
        person("Felix G.M.", "Ernst", role=c("ctb"), email="felix.gm.ernst@outlook.com"),
        person("Alan", "O'Callaghan", role=c("ctb", "cre"), email="alan.ocallaghan@outlook.com"),
        person("Yun", "Peng", role=c("ctb"), email="yunyunp96@gmail.com"),
        person("Leo", "Lahti", role=c("ctb"), email="leo.lahti@utu.fi", comment = c(ORCID = "0000-0001-5537-637X")),
        person("Tuomas", "Borman", role = c("ctb"), comment = c(ORCID = "0000-0002-8563-8884"))
    )
Version: 1.41.2
Date: 2026-06-30
License: GPL-3
Title: Single-Cell Analysis Toolkit for Gene Expression Data in R
Description: A collection of tools for doing various analyses of
        single-cell RNA-seq gene expression data, with a focus on
        quality control and visualization.
Depends: SingleCellExperiment, scuttle, ggplot2
Imports: stats, utils, methods, Matrix, BiocGenerics, S4Vectors,
        SummarizedExperiment, MatrixGenerics, SparseArray,
        DelayedArray, beachmat, BiocNeighbors, BiocSingular,
        BiocParallel, rlang, ggbeeswarm, viridis, Rtsne, RColorBrewer,
        RcppML, uwot, pheatmap, ggrepel
Suggests: BiocStyle, DelayedMatrixStats, snifter, densvis, cowplot,
        biomaRt, knitr, scRNAseq, robustbase, rmarkdown, testthat,
        Biobase, scattermore, ggrastr, MASS
VignetteBuilder: knitr
biocViews: ImmunoOncology, SingleCell, RNASeq, QualityControl,
        Preprocessing, Normalization, Visualization,
        DimensionReduction, Transcriptomics, GeneExpression,
        Sequencing, Software, DataImport, DataRepresentation,
        Infrastructure, Coverage
Encoding: UTF-8
URL: http://bioconductor.org/packages/scater/
BugReports: https://support.bioconductor.org/
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/scater
git_branch: devel
git_last_commit: b941749
git_last_commit_date: 2026-06-30
Repository: Bioconductor 3.24
Date/Publication: 2026-06-30
NeedsCompilation: no
Packaged: 2026-07-01 00:05:04 UTC; biocbuild
Author: Davis McCarthy [aut],
  Kieran Campbell [aut],
  Aaron Lun [aut, ctb],
  Quin Wills [aut],
  Vladimir Kiselev [ctb],
  Felix G.M. Ernst [ctb],
  Alan O'Callaghan [ctb, cre],
  Yun Peng [ctb],
  Leo Lahti [ctb] (ORCID: <https://orcid.org/0000-0001-5537-637X>),
  Tuomas Borman [ctb] (ORCID: <https://orcid.org/0000-0002-8563-8884>)
Maintainer: Alan O'Callaghan <alan.ocallaghan@outlook.com>
