# Generated by roxygen2: do not edit by hand

export("bootstraps<-")
export("fpkm<-")
export("norm_exprs<-")
export("stand_exprs<-")
export(addPerCellQC)
export(addPerFeatureQC)
export(aggregateAcrossCells)
export(aggregateAcrossFeatures)
export(annotateBMFeatures)
export(bootstraps)
export(calculateAverage)
export(calculateCPM)
export(calculateDiffusionMap)
export(calculateFPKM)
export(calculateMDS)
export(calculateMultiUMAP)
export(calculateNMF)
export(calculatePCA)
export(calculateQCMetrics)
export(calculateTPM)
export(calculateTSNE)
export(calculateUMAP)
export(categoricalHlines)
export(categoricalHlinesNamed)
export(centreSizeFactors)
export(computeLibraryFactors)
export(computeMedianFactors)
export(fpkm)
export(getBMFeatureAnnos)
export(getExplanatoryPCs)
export(getVarianceExplained)
export(ggcells)
export(ggfeatures)
export(isOutlier)
export(librarySizeFactors)
export(logNormCounts)
export(makePerCellDF)
export(makePerFeatureDF)
export(medianSizeFactors)
export(mockSCE)
export(multiplot)
export(nexprs)
export(norm_exprs)
export(normalizeCounts)
export(numDetectedAcrossCells)
export(numDetectedAcrossFeatures)
export(perCellQCMetrics)
export(perFeatureQCMetrics)
export(plotColData)
export(plotDiffusionMap)
export(plotDots)
export(plotExplanatoryPCs)
export(plotExplanatoryVariables)
export(plotExpression)
export(plotGroupedHeatmap)
export(plotHeatmap)
export(plotHighestExprs)
export(plotMDS)
export(plotNMF)
export(plotPCASCE)
export(plotPlatePosition)
export(plotRLE)
export(plotReducedDim)
export(plotRowData)
export(plotScater)
export(plotTSNE)
export(plotUMAP)
export(plotXY)
export(projectReducedDim)
export(quickPerCellQC)
export(readSparseCounts)
export(retrieveCellInfo)
export(retrieveFeatureInfo)
export(runColDataPCA)
export(runDiffusionMap)
export(runMDS)
export(runMultiUMAP)
export(runNMF)
export(runTSNE)
export(runUMAP)
export(stand_exprs)
export(sumCountsAcrossCells)
export(sumCountsAcrossFeatures)
export(toSingleCellExperiment)
export(uniquifyFeatureNames)
export(updateSCESet)
exportClasses(SCESet)
exportMethods(bootstraps)
exportMethods(calculateDiffusionMap)
exportMethods(calculateMDS)
exportMethods(calculateMultiUMAP)
exportMethods(calculateNMF)
exportMethods(calculatePCA)
exportMethods(calculateTSNE)
exportMethods(calculateUMAP)
exportMethods(getVarianceExplained)
exportMethods(nexprs)
exportMethods(normalize)
exportMethods(plotPCA)
exportMethods(projectReducedDim)
exportMethods(runPCA)
import(SingleCellExperiment)
import(methods)
import(scuttle)
importClassesFrom(BiocParallel,MulticoreParam)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(SparseArray,COO_SparseMatrix)
importClassesFrom(SparseArray,SVT_SparseMatrix)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(BiocGenerics,"counts<-")
importFrom(BiocGenerics,colnames)
importFrom(BiocGenerics,counts)
importFrom(BiocGenerics,normalize)
importFrom(BiocGenerics,plotPCA)
importFrom(BiocGenerics,rownames)
importFrom(BiocNeighbors,KmknnParam)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpnworkers)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(BiocSingular,ExactParam)
importFrom(BiocSingular,bsparam)
importFrom(BiocSingular,runPCA)
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,getAutoBPPARAM)
importFrom(DelayedArray,setAutoBPPARAM)
importFrom(Matrix,t)
importFrom(MatrixGenerics,colQuantiles)
importFrom(MatrixGenerics,colSums)
importFrom(MatrixGenerics,colSums2)
importFrom(MatrixGenerics,colVars)
importFrom(MatrixGenerics,rowMeans)
importFrom(MatrixGenerics,rowMeans2)
importFrom(MatrixGenerics,rowMedians)
importFrom(MatrixGenerics,rowSums)
importFrom(MatrixGenerics,rowSums2)
importFrom(MatrixGenerics,rowVars)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(SingleCellExperiment,"cpm<-")
importFrom(SingleCellExperiment,"logcounts<-")
importFrom(SingleCellExperiment,"normcounts<-")
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,"reducedDims<-")
importFrom(SingleCellExperiment,"tpm<-")
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,altExpNames)
importFrom(SingleCellExperiment,cpm)
importFrom(SingleCellExperiment,logcounts)
importFrom(SingleCellExperiment,normcounts)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SingleCellExperiment,tpm)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(beachmat,realizeFileBackedMatrix)
importFrom(beachmat,rowBlockApply)
importFrom(ggbeeswarm,geom_quasirandom)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bin2d)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_hex)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_colour_gradient)
importFrom(ggplot2,scale_colour_gradient2)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,stat_density)
importFrom(ggplot2,stat_smooth)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,stat_summary_2d)
importFrom(ggplot2,stat_summary_hex)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_text_repel)
importFrom(methods,as)
importFrom(rlang,as_label)
importFrom(scuttle,.bpNotSharedOrUp)
importFrom(scuttle,addPerCellQC)
importFrom(scuttle,addPerFeatureQC)
importFrom(scuttle,aggregateAcrossCells)
importFrom(scuttle,aggregateAcrossFeatures)
importFrom(scuttle,calculateAverage)
importFrom(scuttle,calculateCPM)
importFrom(scuttle,calculateFPKM)
importFrom(scuttle,calculateTPM)
importFrom(scuttle,computeLibraryFactors)
importFrom(scuttle,computeMedianFactors)
importFrom(scuttle,fitLinearModel)
importFrom(scuttle,isOutlier)
importFrom(scuttle,librarySizeFactors)
importFrom(scuttle,logNormCounts)
importFrom(scuttle,makePerCellDF)
importFrom(scuttle,makePerFeatureDF)
importFrom(scuttle,medianSizeFactors)
importFrom(scuttle,mockSCE)
importFrom(scuttle,normalizeCounts)
importFrom(scuttle,numDetectedAcrossCells)
importFrom(scuttle,numDetectedAcrossFeatures)
importFrom(scuttle,perCellQCMetrics)
importFrom(scuttle,perFeatureQCMetrics)
importFrom(scuttle,quickPerCellQC)
importFrom(scuttle,readSparseCounts)
importFrom(scuttle,sumCountsAcrossCells)
importFrom(scuttle,sumCountsAcrossFeatures)
importFrom(scuttle,uniquifyFeatureNames)
importFrom(stats,cmdscale)
importFrom(stats,dist)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(viridis,scale_colour_viridis)
importFrom(viridis,scale_fill_viridis)
importFrom(viridis,viridis)
importMethodsFrom(DelayedArray,sweep)
