#' The \pkg{scater} package
#'
#' Provides functions for convenient visualization of single-cell data,
#' mostly via \pkg{ggplot2}.
#' It also used to provide utilities for data transformation and quality control,
#' but these have largely been moved to the \pkg{scuttle} package.
#'
#' @author Davis McCarthy, Aaron Lun
#'
#' @name scater-pkg
#' @import scuttle
#' @import SingleCellExperiment
#' @import methods
NULL
